@@ -1,24 +1,23 @@
 import numpy as np
 import pandas as pd
 import seaborn as sns
-from matplotlib import pyplot, colors, cm
-from mpl_toolkits.mplot3d import art3d
+from matplotlib import pyplot
 from numpy import linalg as lin
 
 from hikari.dataframes import HklFrame
 from hikari.symmetry import SG, Group
-from hikari.utility import make_abspath, mpl_map_palette, gnuplot_map_palette, \
-    fibonacci_sphere, rotation_around
-from hikari.resources import potency_map_template
+from hikari.utility import make_abspath, fibonacci_sphere, rotation_around
+from hikari.scripts.angular_explorer import angular_property_explorer_factory
 
 
 def potency_map(a, b, c, al, be, ga,
                 space_group='P1',
                 axis='',
                 fix_scale=False,
+                histogram=True,
                 opening_angle=35,
-                output_directory='~',
-                output_name='cplt_map',
+                orientation=None,
+                path='~/sortav.lst',
                 output_quality=3,
                 resolution=1.2,
                 wavelength='MoKa'):
@@ -27,10 +26,9 @@ def potency_map(a, b, c, al, be, ga,
     (DAC) with a given opening angle, as a function of crystal orientation.
     For details see `this paper <https://doi.org/10.1107/S2052252521009532>`_.
 
-    The script accepts unit cell & space group information, and predicts the
+    The script accepts unit cell & space group information, and predicts max
     completeness of fully merged data for investigated crystal in said group.
-    Results are logged into text files and drawn with gnuplot or matplotlib,
-    depending on settings.
+    Results are logged into text files and drawn with gnuplot or matplotlib.
 
     Potency is calculated and visualised on a unit-sphere orientation heatmap.
     Each point **p** is associated with a certain crystal orientation in DAC,
@@ -41,7 +39,7 @@ def potency_map(a, b, c, al, be, ga,
     symmetry, only a representative part of sphere (usually an octant) is shown.
 
     As an example, let's assume a orthorhombic cell with *a* = *b* = *c* = 10
-    and laue group "mmm". Running the script and generating completeness the map
+    and laue group "mmm". Running the script and generating the potency the map
     yields the lowest values close to **X\***, **Y\*** and **Z\*** vector,
     while the highest values are observed between those vectors.
     Placing the crystal on its [100] face inside the dac will cause the
@@ -92,20 +90,24 @@ def potency_map(a, b, c, al, be, ga,
     :param ga: Unit cell parameter *alpha* in degrees.
     :type ga: float
     :param space_group: Short Hermann-Mauguin name or index of space group.
-        For details see table in hikari.symmetry.space_groups.
+        For details see :py:mod:`hikari.symmetry.space_groups`.
     :type space_group: str or int
     :param axis: domain to calculate potency in. Accepts 'x'/ 'y'/ 'z' for h00/
         0k0/ 00l, 'xy'/'xz'/'yz' for hk0/ h0l/ 0kl, or '' for all reflections.
     :type axis: string
     :param fix_scale: If true, the colour scheme will fix to 0 - 100% range.
     :type fix_scale: bool
+    :param histogram: If true, potency distribution will be plotted as histogram
+    :type histogram: bool
     :param opening_angle: Value of single opening angle as defined in
         :meth:`hikari.dataframes.HklFrame.dac`.
     :type opening_angle: float
-    :param output_directory: Path to directory where output should be saved.
-    :type output_directory: str
-    :param output_name: Base name for files created in `output_directory`.
-    :type output_name: str
+    :param orientation: either a cif-style 3x3 matrix of crystal orientation or
+        a 3-length array with a diamond-perpendicular face to be marked on a map
+    :type orientation: np.ndarray
+    :param path: Path to a file where script is to be run. Extension is ignored,
+        but file name will and must be the same for all input and output files.
+    :type path: str
     :param output_quality: Density of individual orientations to be considered.
         Should be in range from 1 (every 15 degrees) to 5 (every 1 degree).
     :type output_quality: int
@@ -117,215 +119,13 @@ def potency_map(a, b, c, al, be, ga,
     :return: None
     :rtype: None
     """
-    dat_path = make_abspath(output_directory, output_name + '.dat')
-    gnu_path = make_abspath(output_directory, output_name + '.gnu')
-    lst_path = make_abspath(output_directory, output_name + '.lst')
-    png_path = make_abspath(output_directory, output_name + '.png')
-    axis = axis.lower()
-    sg = SG[space_group]
-    lg = sg.reciprocate()
-
-    def _make_hkl_frame(ax=axis):
-        """Make ball or axis of hkl which will be cut in further steps"""
-        _f = HklFrame()
-        _f.edit_cell(a=a, b=b, c=c, al=al, be=be, ga=ga)
-        _f.la = wavelength
-        _f.fill(radius=min(_f.r_lim, resolution))
-        if ax in {'x'}:
-            _f.table = _f.table.loc[_f.table['k'].eq(0) & _f.table['l'].eq(0)]
-        elif ax in {'y'}:
-            _f.table = _f.table.loc[_f.table['h'].eq(0) & _f.table['l'].eq(0)]
-        elif ax in {'z'}:
-            _f.table = _f.table.loc[_f.table['h'].eq(0) & _f.table['k'].eq(0)]
-        elif ax in {'xy'}:
-            _f.table = _f.table.loc[_f.table['l'].eq(0)]
-        elif ax in {'xz'}:
-            _f.table = _f.table.loc[_f.table['k'].eq(0)]
-        elif ax in {'yz'}:
-            _f.table = _f.table.loc[_f.table['h'].eq(0)]
-        if ax in {'x', 'y', 'z', 'xy', 'xz', 'yz'}:
-            _f.transform([o.tf for o in lg.operations])
-        _f.extinct(sg)
-        return _f
-
-    p = _make_hkl_frame()
-    p.find_equivalents(point_group=lg)
-    total_reflections = p.table['equiv'].nunique()
-    if total_reflections == 0:
-        raise KeyError('Specified part of reciprocal space contains zero nodes')
-
-    def _determine_theta_and_phi_limits():
-        """Define range of coordinates where potency map will be calculated.
-        v1, v2, v3 are normal vectors pointing in zenith direction z*,
-        orthogonal direction (x) and direction perpendicular to them both."""
-        _v1 = p.c_w / lin.norm(p.c_w)
-        _v2 = p.a_v / lin.norm(p.a_v)
-        _v3 = np.cross(_v1, _v2)
-
-        if sg.system in {Group.System.triclinic, Group.System.monoclinic,
-                         Group.System.orthorhombic, Group.System.tetragonal,
-                         Group.System.cubic}:
-            _th_limits = [0, 90]
-            _ph_limits = [0, 90]
-        elif sg.system in {Group.System.trigonal,
-                           Group.System.hexagonal}:
-            _th_limits = [0, 90]
-            _ph_limits = [0, 120]
-        else:
-            raise ValueError('Unknown crystal system (trigonal not supported)')
-        return _v1, _v2, _v3, _th_limits, _ph_limits
-
-    v1, v2, v3, th_limits, ph_limits = _determine_theta_and_phi_limits()
-
-    def _make_theta_and_phi_mesh():
-        """Define a list of theta and phi values to be investigated."""
-        if output_quality not in {1, 2, 3, 4, 5}:
-            raise KeyError('output_quality should be 1, 2, 3, 4 or 5')
-        angle_res = {1: 15, 2: 10, 3: 5, 4: 2, 5: 1}[output_quality]
-        _th_range = np.arange(th_limits[0], th_limits[1] + 0.001, angle_res)
-        _ph_range = np.arange(ph_limits[0], ph_limits[1] + 0.001, angle_res)
-        _th_mesh, _ph_mesh = np.meshgrid(_th_range, _ph_range)
-        return _th_range, _ph_range, _th_mesh, _ph_mesh
-
-    th_range, ph_range, th_mesh, ph_mesh = _make_theta_and_phi_mesh()
-    data_dict = {'th': [], 'ph': [], 'cplt': [], 'reflns': []}
-
-    def _angles_to_vector(theta, phi):
-        """Find the vector by rotating v1 by theta and then phi, in degrees."""
-        _sin_th = np.sin(np.deg2rad(theta))
-        _sin_ph = np.sin(np.deg2rad(phi))
-        _cos_th = np.cos(np.deg2rad(theta))
-        _cos_ph = np.cos(np.deg2rad(phi))
-        _v = v1 * _cos_th + v2 * _sin_th
-        _v_parallel = np.dot(_v, v1) * v1
-        _v_perpend = lin.norm(_v - _v_parallel) * (_cos_ph * v2 + _sin_ph * v3)
-        return _v_parallel + _v_perpend
-
-    def _calculate_completeness_mesh():
-        """Calculate completeness for each individual pair of theta and phi."""
-        _cplt_mesh = np.zeros_like(th_mesh)
-        lst = open(lst_path, 'w+')
-        lst.write('#     th      ph    cplt  reflns\n')
-        vectors = np.vstack([_angles_to_vector(th, ph) for th in th_range
-                                             for ph in ph_range])
-        uniques = p.dacs_count(opening_angle=opening_angle, vectors=vectors)
-        for i, th in enumerate(th_range):
-            for j, ph in enumerate(ph_range):
-                count = uniques[i * len(ph_range) + j]
-                potency = count / total_reflections
-                data_dict['th'].append(th)
-                data_dict['ph'].append(ph)
-                data_dict['cplt'].append(potency)
-                data_dict['reflns'].append(count)
-                lst.write(f'{th:8.0f}{ph:8.0f}{potency:8.5f}{count:8d}\n')
-                _cplt_mesh[j][i] = potency
-            lst.write('\n')
-        index_max = np.unravel_index(np.argmax(_cplt_mesh), _cplt_mesh.shape)
-        best_th, best_ph = th_range[index_max[1]], ph_range[index_max[0]]
-        index_min = np.unravel_index(np.argmin(_cplt_mesh), _cplt_mesh.shape)
-        worst_th, worst_ph = th_range[index_min[1]], ph_range[index_min[0]]
-        max_p = max(data_dict['cplt'])
-        min_p = min(data_dict['cplt'])
-        mean_p = np.mean(data_dict['cplt'])
-        lst.write(f'# best cplt = {max_p} for th = {best_th}, ph = {best_ph}\n'
-                  f'# worst cplt = {min_p} for th = {worst_th}, ph = {worst_ph}'
-                  f'\n# mean cplt = {mean_p}\n')
-        lst.close()
-        np.savetxt(dat_path, _cplt_mesh)
-        return data_dict, _cplt_mesh
-
-    data_dict, cplt_mesh = _calculate_completeness_mesh()
-
-    def _plot_in_matplotlib():
-        """Plot the completeness map in radial coordinates using matplotlib"""
-        fig = pyplot.figure(figsize=(5, 3))
-        ax = fig.add_subplot(111, projection='3d')
-        ax.view_init(elev=90 - sum(th_limits) / 2, azim=sum(ph_limits) / 2)
-        ax.dist = 10
-        ax.set_axis_off()
-
-        # surface in cartesian coordinates
-        x = np.sin(np.deg2rad(th_mesh)) * np.cos(np.deg2rad(ph_mesh))
-        y = np.sin(np.deg2rad(th_mesh)) * np.sin(np.deg2rad(ph_mesh))
-        z = np.cos(np.deg2rad(th_mesh))
-
-        # wireframe
-        np.warnings.filterwarnings('ignore',
-                                   category=np.VisibleDeprecationWarning)
-        ax.plot_wireframe(x, y, z, colors='k', linewidth=0.25)  # <- mpl warning
-
-        # color map
-        my_heatmap_colors = mpl_map_palette[axis]
-        my_colormap = colors.LinearSegmentedColormap.from_list(
-            'heatmapEX', my_heatmap_colors, N=256)
-        m = cm.ScalarMappable(cmap=my_colormap)
-        m.set_array(cplt_mesh)
-        if fix_scale is True:
-            m.set_clim(0, 1)
-            norm = colors.Normalize(vmin=0, vmax=1)
-        else:
-            norm = colors.Normalize(vmin=min(data_dict['cplt']),
-                                    vmax=max(data_dict['cplt']))
-        pyplot.colorbar(m, fraction=0.046, pad=0.04)
-
-        # direction lines
-        _len = 1.25
-        _x = p.a_w / lin.norm(p.a_w)
-        _y = p.b_w / lin.norm(p.b_w)
-        _z = p.c_w / lin.norm(p.c_w)
-        ax.add_line(art3d.Line3D((_x[0], _len * _x[0]), (_x[1], _len * _x[1]),
-                                 (_x[2], _len * _x[2]), color='r', linewidth=5))
-        ax.add_line(art3d.Line3D((_y[0], _len * _y[0]), (_y[1], _len * _y[1]),
-                                 (_y[2], _len * _y[2]), color='g', linewidth=5))
-        ax.add_line(art3d.Line3D((_z[0], _len * _z[0]), (_z[1], _len * _z[1]),
-                                 (_z[2], _len * _z[2]), color='b', linewidth=5))
-
-        # color mesh for heatmap
-        color_mesh = cplt_mesh[:-1, :-1]
-        for i in range(cplt_mesh.shape[0] - 1):
-            for j in range(cplt_mesh.shape[1] - 1):
-                color_mesh[i, j] = (cplt_mesh[i + 1, j] + cplt_mesh[i, j + 1] +
-                                    cplt_mesh[i, j] + cplt_mesh[
-                                        i + 1, j + 1]) / 4
-
-        # heatmap surface
-        for item in [fig, ax]:
-            item.patch.set_visible(False)
-        ax.plot_surface(x, y, z, rstride=1, cstride=1, cmap=my_colormap,
-                        linewidth=0, antialiased=False,
-                        facecolors=my_colormap(norm(color_mesh)))
-        pyplot.savefig(png_path, dpi=600, format='png', bbox_inches=None)
-    _plot_in_matplotlib()
-
-    def _prepare_gnuplot_input():
-        """Prepare input to completeness map in radial coordinates in gnuplot"""
-        gnu = open(gnu_path, 'w+')
-        gnu.write(potency_map_template.format(
-            axis_x1=(p.a_w / lin.norm(p.a_w))[0],
-            axis_x2=(p.a_w / lin.norm(p.a_w))[1],
-            axis_x3=(p.a_w / lin.norm(p.a_w))[2],
-            axis_y1=(p.b_w / lin.norm(p.b_w))[0],
-            axis_y2=(p.b_w / lin.norm(p.b_w))[1],
-            axis_y3=(p.b_w / lin.norm(p.b_w))[2],
-            axis_z1=(p.c_w / lin.norm(p.c_w))[0],
-            axis_z2=(p.c_w / lin.norm(p.c_w))[1],
-            axis_z3=(p.c_w / lin.norm(p.c_w))[2],
-            cplt_min=0 if fix_scale else min(data_dict['cplt']) * 100,
-            cplt_max=100 if fix_scale else max(data_dict['cplt']) * 100,
-            job_name=output_name,
-            min_ph=min(ph_limits),
-            max_ph=max(ph_limits),
-            min_th=min(th_limits),
-            max_th=max(th_limits),
-            palette=gnuplot_map_palette[axis.lower()]))
-
-    _prepare_gnuplot_input()
-    try:
-        from os import system, getcwd
-        _path = make_abspath(output_directory)
-        system('cd ' + _path + '; gnuplot ' + gnu_path)
-    except OSError:
-        pass
+    kwargs = locals()
+    ape = angular_property_explorer_factory.create(prop='potency')
+    ape.set_up(**kwargs)
+    ape.explore()
+    ape.write_hist_file()
+    ape.draw_matplotlib_map()
+    ape.draw_gnuplot_map()
 
 
 def potency_vs_dac_opening_angle(output_path='~/output.txt',
@@ -334,7 +134,7 @@ def potency_vs_dac_opening_angle(output_path='~/output.txt',
                                  wavelength='MoKa',
                                  theta=None):
     """
-    Calculate completeness in P1 as a function of DAC opening angle,
+    Calculate potency in P1 space group as a function of DAC opening angle,
     assuming certain resolution and wavelength used.
 
     :param output_path: Path of created file containing calculated data.
@@ -363,15 +163,14 @@ def potency_vs_dac_opening_angle(output_path='~/output.txt',
         return hkl_frame
 
     p = _make_reference_ball()
-    v = fibonacci_sphere(10)
+    v = fibonacci_sphere(100)
     total = len(p)
     angles = np.linspace(start=90, stop=0, num=precision)
-    out = open(make_abspath(output_path), 'w')
-    out.write('#oa      cplt\n')
-    for a in angles:  # for one random vector v
-        c = p.dacs_count(opening_angle=a, vectors=v)
-        out.write(' {a:7.4f} {c:7.5f}\n'.format(a=a, c=np.mean(c)/total))
-    out.close()
+    with open(make_abspath(output_path), 'w', buffering=1) as out:
+        out.write('#     oa potency\n')
+        for a in angles:  # for one random vector v
+            c = p.dacs_count(opening_angle=a, vectors=v)
+            out.write(f' {a:7.4f} {np.mean(c)/total:7.5f}\n')
 
 
 laue_space_groups = 'P-1', 'P2/m', 'Pmmm', 'P4/m', 'P4/mmm', 'P-3', 'P-3m1',\
@@ -507,10 +306,10 @@ def dac_potency_around_axis(a, b, c, al, be, ga,
                             opening_angle=35.0,
                             wavelength='MoKa',
                             vector=(1, 0, 0),
-                            topple=5):
+                            topple_angle=5):
     """
-    For a given simulated .hkl file calculate average completeness of data
-    obtained by toppling the crystal by "topple" degrees from the axis.
+    For a given crystal, opening angle, and wavelength calculate average potency
+    obtained by toppling the crystal by "topple_angle"° from the "vector" axis.
 
     :param a: Unit cell parameter *a* in Angstrom.
     :type a: float
@@ -525,22 +324,22 @@ def dac_potency_around_axis(a, b, c, al, be, ga,
     :param ga: Unit cell parameter *gamma* in degrees.
     :type ga: float
     :param space_group: Short Hermann-Mauguin name or index of space group.
-        For details see table in hikari.symmetry.space_groups.
+        For details see :py:mod:`hikari.symmetry.space_groups`.
     :type space_group: str or int
     :param wavelength: Wavelength of radiation utilised in experiment.
     :type wavelength: float or str
     :param opening_angle: Value of single opening angle as defined in
         :meth:`hikari.dataframes.HklFrame.dac`.
     :type opening_angle: float
-    :param vector: Direction around which completeness will be calculated.
+    :param vector: Direction from which a theoretical crystal will be toppled.
     :type vector: tuple
-    :param topple: Angle by which vector will be toppled in varous directions.
-    :type topple: float
+    :param topple_angle: Angle by which theoretical crystal will be toppled.
+    :type topple_angle: float
     :return: None
     """
 
     sg = SG[space_group]
-    pg = sg.reciprocate()  # .lauefy()
+    pg = sg.reciprocate()  # .lauefy()  # uncomment if hkl and -h-k-l are equiv.
 
     p = HklFrame()
     p.edit_cell(a=a, b=b, c=c, al=al, be=be, ga=ga)
@@ -563,21 +362,21 @@ def dac_potency_around_axis(a, b, c, al, be, ga,
         return _v @ rotation_around(_k, by=np.deg2rad(angle))
 
     # generate 360 toppled vectors
-    toppled = rotate(v, perp, topple)
-    toppleds = [rotate(toppled, v, i) for i in range(360)]
+    toppled_vector = rotate(v, perp, topple_angle)
+    toppled_vectors = [rotate(toppled_vector, v, i) for i in range(360)]
 
-    # calculate the completeness for toppleds
+    # calculate the potency for toppleds
     rads = [2.00, 1/0.7, 1.20, 1.00, 1/1.5]
     cplt = [0] * len(rads)
     p.trim(max(rads))
-    for t in toppleds:
+    for t in toppled_vectors:
         q = p.copy()
         q.dac_trim(opening_angle=opening_angle, vector=t)
         for cplt_bin, rad in enumerate(rads, start=1):
             q.trim(rad)
             cplt[-cplt_bin] += q.table['equiv'].nunique()
 
-    # divide lists of completeness by total and return from sums to individuals
+    # divide lists of potency by total and return from sums to individuals
     full = [0] * len(rads)
     for cplt_bin, rad in enumerate(rads, start=1):
         p.trim(rad)
@@ -588,13 +387,12 @@ def dac_potency_around_axis(a, b, c, al, be, ga,
     full = [f2 - f1 for f1, f2 in zip([0] + full[:-1], full)]
     cplt = [c / (360 * f) for c, f in zip(cplt, full)]
 
-    print('Resolution limits (in distance to reflection, twice reciprocal:')
+    print('Resolution limits in distance to reflection, twice sin(θ/λ):')
     print(np.array(list(reversed(rads))))
-    print('Average shell completeness with {}degree topple'.format(topple))
+    print(f'Average shell potency with {topple_angle} degree topple')
     print(np.array(cplt))
 
 
 if __name__ == '__main__':
-    # potency_map(10, 10, 10, 90, 90, 90, space_group='Pmmm',
-    #             resolution=1.2, output_directory='~/_/', output_name='_')
-    pass
+    potency_map(a=10, b=10, c=10, al=90, be=90, ga=120, space_group='P21/c',
+                path='~/_/potency.hkl', output_quality=5, wavelength='MoKa')
