@@ -26,16 +26,6 @@ U80 = uncertainties.ufloat(80.0, 1.0)
 U90 = uncertainties.ufloat(90.0, 1.0)
 
 
-class TempFile:
-    temp_dir = tempfile.TemporaryDirectory()
-
-    def __init__(self, name, content):
-        self.name = name
-        self.path = str(pathlib.Path(self.temp_dir.name) / name)
-        self.file = open(self.path, 'w+')
-        self.file.write(content)
-
-
 nacl_cif_path = str(pathlib.Path(__file__).parent.joinpath('NaCl.cif'))
 nacl_fcf_path = str(pathlib.Path(__file__).parent.joinpath('NaCl.fcf'))
 nacl_hkl_path = str(pathlib.Path(__file__).parent.joinpath('NaCl.hkl'))
@@ -111,9 +101,34 @@ class TestBaseFrame(unittest.TestCase):
         self.assertAlmostEqual(self.b.v_r, 0.003657181855591761)
 
 
-class TestCifBlockRead(unittest.TestCase):
+class TestCifFrameReader(unittest.TestCase):
+    def setUp(self) -> None:
+        self.c = CifFrame()
+
+    def test_read_cif_file(self):
+        self.c.read(nacl_cif_path)
+        self.assertIn('NaCl', self.c)
+        self.assertIsInstance(self.c['NaCl'], CifBlock)
+
+    def test_read_fcf_file(self):
+        self.c.read(nacl_fcf_path)
+        self.assertIn('NaCl', self.c)
+        self.assertIsInstance(self.c['NaCl'], CifBlock)
+
+
+class TestCifBlockReader(unittest.TestCase):
+    def setUp(self) -> None:
+        self.b = CifBlock()
+
     def test_read(self):
-        CifBlock().read(path=nacl_cif_path, block='NaCl')
+        self.b.read(path=nacl_cif_path, block='NaCl')
+        self.assertIn('_audit_creation_date', self.b)
+        self.assertIsInstance(self.b, CifBlock)
+
+    def test_read_loop(self):
+        self.b.read(nacl_cif_path, 'NaCl')
+        self.assertEqual(self.b['_atom_type_symbol'], ['Cl', 'Na'])
+        self.assertEqual(len(self.b['_space_group_symop_operation_xyz']), 192)
 
 
 class TestCifBlockGeneral(unittest.TestCase):
@@ -121,17 +136,34 @@ class TestCifBlockGeneral(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls) -> None:
-        super().setUpClass()
         cls.b.read(path=nacl_cif_path, block='NaCl')
 
-    def test_access(self):
-        self.assertIn('_cell_length_a', self.b)
+    def test_access_existing(self):
         self.assertEqual(self.b['_cell_length_a'], '5.64109(5)')
         self.assertEqual(len(self.b['_diffrn_refln_index_h']), 8578)
 
+    def test_access_nonexistent(self):
+        with self.assertRaises(KeyError):
+            _ = self.b['_nonexistent_key']
+
     def test_assign(self):
         self.b['_cell_angle_beta'] = '120'
         self.assertEqual(self.b['_cell_angle_beta'], '120')
+        self.b['_fictitious_list'] = ['f', 'i', 'c']
+        self.assertEqual(self.b['_fictitious_list'], ['f', 'i', 'c'])
+
+    def test_contains(self):
+        self.assertIn('_cell_length_a', self.b)
+        self.assertNotIn('_nonexistent_key', self.b)
+
+    def test_get(self):
+        self.assertEqual(self.b.get('_cell_length_a'), '5.64109(5)')
+
+    def test_get_nonexistent(self):
+        self.assertIs(self.b.get('_nonexistent_key'), None)
+
+    def test_get_with_default(self):
+        self.assertEqual(self.b.get('_nonexistent_key', 'default'), 'default')
 
     def test_get_as_type_single(self):
         k = '_cell_angle_alpha'
@@ -147,18 +179,17 @@ class TestCifBlockGeneral(unittest.TestCase):
         u_typ = uncertainties.ufloat_fromstr
         self.assertEqual(repr(self.b.get_as_type(k, typ=u_typ)), repr([U1, U1]))
 
+    def test_get_as_type_nonexistent(self):
+        self.assertIs(self.b.get_as_type('_nonexistent_key', str), None)
+
     def test_get_as_type_exceptions(self):
-        with self.assertRaises(KeyError):
-            _ = self.b.get_as_type('_nonexistent_key', str)
-        self.b['_cell_angle_beta'] = U90
+        self.b['_cell_angle_beta'] = U90  # CifBlock stores str and list types
         with self.assertRaises(TypeError):
             _ = self.b.get_as_type('_cell_angle_beta', str)
 
     def test_get_as_type_defaults(self):
-        self.assertEqual(
-            self.b.get_as_type('_nonexistent_key_with_default', str, 'default'),
-            'default'
-        )
+        self.assertEqual(self.b.get_as_type('_nonexistent_key', str, 'default'),
+                         'default')
 
 
 class TestCifBlockInterface(unittest.TestCase):
@@ -179,25 +210,62 @@ class TestCifBlockInterface(unittest.TestCase):
 
 
 class TestCifValidator(unittest.TestCase):
-    def test_validator(self):
-        c = CifValidator()
-        self.assertIn('_atom_site_label', c)
-        self.assertIsInstance(c['_atom_site_label'], CifBlock)
+    v = CifValidator()
+
+    def test_creation(self):
+        self.assertTrue('_atom_site_label' in self.v)
+        self.assertIn('_atom_site_label', self.v)
+        self.assertIsInstance(self.v.get('_atom_site_label'), CifBlock)
+
+    def test_contains(self):
+        self.assertIn('atom_site_fract_', self.v)
+        self.assertIn('_atom_site_fract_', self.v)
+        self.assertIn('_atom_site_fract_x', self.v)
+
+    def test_get(self):
+        self.assertEqual(self.v.get('atom_site_fract_')['_type'], 'numb')
+        self.assertEqual(self.v.get('_atom_site_fract_')['_type'], 'numb')
+        self.assertEqual(self.v.get('_atom_site_fract_x')['_type'], 'numb')
+        self.assertIs(self.v.get('nonexistent_key'), None)
+
+    def test_get__category(self):
+        cat = 'atom_site'
+        self.assertEqual(self.v.get__category('atom_site_fract_'), cat)
+        self.assertEqual(self.v.get__category('_atom_site_fract_'), cat)
+        self.assertEqual(self.v.get__category('_atom_site_fract_x'), cat)
+        self.assertEqual(self.v.get__category('nonexistent_key', 'def'), 'def')
+
+    def test_get__list(self):
+        self.assertIs(self.v.get__list('atom_site_fract_'), True)
+        self.assertIs(self.v.get__list('refine_ls_number_reflns'), None)
+        self.assertEqual(self.v.get__list('exptl_crystal_colour'), None)
+        self.assertEqual(self.v.get__list('nonexistent_key', 'def'), 'def')
 
 
-class TestCifFrame(unittest.TestCase):
-    c_cif = CifFrame()
-    c_fcf = CifFrame()
+class TestCifWriter(unittest.TestCase):
+    temp_dir = tempfile.TemporaryDirectory()
+    temp_path1 = str(pathlib.Path(temp_dir.name) / 'temp1.cif')
+    temp_path2 = str(pathlib.Path(temp_dir.name) / 'temp2.cif')
 
-    def test_read_cif_file(self):
-        self.c_cif.read(nacl_cif_path)
-        self.assertIn('NaCl', self.c_cif)
-        self.assertIsInstance(self.c_cif['NaCl'], CifBlock)
+    @classmethod
+    def tearDownClass(cls) -> None:
+        cls.temp_dir.cleanup()
 
-    def test_read_fcf_file(self):
-        self.c_fcf.read(nacl_fcf_path)
-        self.assertIn('NaCl', self.c_fcf)
-        self.assertIsInstance(self.c_fcf['NaCl'], CifBlock)
+    def setUp(self) -> None:
+        self.c_cif1 = CifFrame()
+        self.c_cif2 = CifFrame()
+        self.c_cif1.read(nacl_cif_path)
+
+    def test_write_cif_file(self):
+        self.c_cif1.write(self.temp_path1)
+
+    def test_write_cif_file_is_consistent(self):
+        self.c_cif1.write(self.temp_path1)
+        self.c_cif2.read(self.temp_path1)
+        self.c_cif2.write(self.temp_path2)
+        with open(self.temp_path1, 'r') as cif1_contents:
+            with open(self.temp_path2, 'r') as cif2_contents:
+                self.assertEqual(cif1_contents.read(), cif2_contents.read())
 
 
 class TestHklFrame(unittest.TestCase):
